/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <device.h>

// * Slave address of the I2C slave */
#define I2C_SLAVE_BASEADDR  (0x40)
#define I2C_SLAVE_CONFIG    (0x00)
#define I2C_SLAVE_SHUNTVOL  (0x01)
#define I2C_SLAVE_BUSVOL    (0x02)
#define I2C_SLAVE_POWER     (0x03)
#define I2C_SLAVE_CURRENT   (0x04)
#define I2C_SLAVE_CARIB     (0x05)
#define I2C_SLAVE_MASK_EN   (0x06)
#define I2C_SLAVE_ALERT     (0x07)
#define I2C_SLAVE_DIEID     (0xFF)

/* I2C Buffer */
uint8   I2CReadBuf[2];
uint8   I2CWriteBuf[3];

uint32  DivTable[8] = {10000000,1000000,100000,10000,1000,100,10,1};
#define DivNum  8u

void Int2String(char *dest, uint32 src)
{
    char   *ptr;
    char   flag;
    uint16  lpCnt;
    
    ptr = dest;
    flag = 0;
    if ( src == 0 )
    {
        *ptr++ = 48;
        *ptr = 0;
        return;
    }

    for(lpCnt = 0; lpCnt < DivNum; lpCnt++)
    {
        if ( src > DivTable[lpCnt] || flag )
        {
            flag = 1;
            *ptr++ = 48 + (src / DivTable[lpCnt] );
            src = src % DivTable[lpCnt];
        }
    }
    *ptr = 0;    
}    

void Float2String(char *dest, float src)
{
    char   *ptr;
    char   flag;
    uint16  lpCnt;
    uint16  src_int;

    src_int = (uint16)(src * 10.0);
    ptr = dest;
    flag = 0;

    if ( src_int == 0 )
    {
        *ptr++ = 48;
        *ptr = 0;
        return;
    }

    
    for(lpCnt = 0; lpCnt < DivNum-1; lpCnt++)
    {
        if ( src_int > DivTable[lpCnt] || flag )
        {
            flag = 1;
            *ptr++ = 48 + (src_int / DivTable[lpCnt] );
            src_int = src_int % DivTable[lpCnt];
        }
    }
    *ptr++ = '.';
    *ptr++ = 48 + src_int;
    *ptr = 0;    
}    


// I2C経由問い合わせルーチン
uint32  InquireI2C(uint8 reg)
{
    // I2C Slaveにレジスタ番号を送信
    I2CWriteBuf[0] = reg;
    I2C_I2CMasterWriteBuf(I2C_SLAVE_BASEADDR, (uint8 *)&I2CWriteBuf[0],
                          01u, I2C_I2C_MODE_COMPLETE_XFER);

    // Slaveの受信待ち
    while(0u == (I2C_I2CMasterStatus() & I2C_I2C_MSTAT_WR_CMPLT));
    
    // I2C Masterのステータスクリア
    (void)I2C_I2CMasterClearStatus();

    // I2C Slaveより結果受信
    I2C_I2CMasterReadBuf(I2C_SLAVE_BASEADDR, (uint8 *)&I2CReadBuf[0],
                         02u, I2C_I2C_MODE_COMPLETE_XFER);

    // 受信完了まで待機
    while(0u == (I2C_I2CMasterStatus() & I2C_I2C_MSTAT_RD_CMPLT));
    
    // I2C Masterのステータスクリア
    (void)I2C_I2CMasterClearStatus();

    return (uint32)(I2CReadBuf[0]*256+I2CReadBuf[1]);
}

// I2C経由Calibration値設定ルーチン
void  CalibrateI2C()
{
   
    // I2C Slaveにレジスタ番号を送信
    I2CWriteBuf[0] = I2C_SLAVE_CARIB;
    I2CWriteBuf[1] = 0x05;
    I2CWriteBuf[2] = 0x00;

    I2C_I2CMasterWriteBuf(I2C_SLAVE_BASEADDR, (uint8 *)I2CWriteBuf,
                          03u, I2C_I2C_MODE_COMPLETE_XFER);

    // Slaveの受信待ち
    while(0u == (I2C_I2CMasterStatus() & I2C_I2C_MSTAT_WR_CMPLT));
    
    // I2C Masterのステータスクリア
    (void)I2C_I2CMasterClearStatus();
}


void main()
{
    float   Temp;               // 温度
    uint32  Vol;                // 電圧
    uint32  Amp;                // 電流
    uint32  Pow;                // 電力
    uint8   Mode;               // 0:表示しない 1:温度表示 2:電圧/電流/電力表示
    uint8   UARTMode;           // 0:送出しない 1:送出する

    char    CharBuf[16];        // 文字列出力用バッファ
    char    CharBuf2[16];       // 文字列出力用バッファ
    char    CharBuf3[16];       // 文字列出力用バッファ

    Mode = 0;
    UARTMode = 0;

    LCD_Start();                // LCD初期化
    UART_Start();               // UART初期化

    ADC_SAR_Seq_Start();        // ADC初期化
    ADC_SAR_Seq_StartConvert(); // ADCサンプリング開始

    // Enable Global Interrupts
    CyGlobalIntEnable;
    
    CapSense_Start();                   // CapSense初期化
    CapSense_InitializeAllBaselines();  // Baseline初期化

    // I2C Master Start
    I2C_Start();                 // I2C Master初期化
    CalibrateI2C();              // 電力測定センサー初期化
    
    for(;;)
    {
        switch( Mode )
        {
            case 1:
                // ADCからデータ読み込み
                ADC_SAR_Seq_IsEndConversion(ADC_SAR_Seq_WAIT_FOR_RESULT);
                Temp = (float)ADC_SAR_Seq_GetResult16(0);
                Temp = Temp * 3300.0 / 4096.0 - 77.0;

                // 文字列に変換
                Float2String(CharBuf, Temp);

                // LCDに出力
                LCD_ClearDisplay();
            	LCD_Position(0u, 0u);
                LCD_PrintString("Temp:");
            	LCD_Position(0u, 5u);
                LCD_PrintString(CharBuf);
                if ( UARTMode )                 
                {
                    // UARTに出力
                	UART_UartPutString(CharBuf);
                    UART_UartPutCRLF(0);   // CR+LF
                }                
                break;
            case 2:
                // I2Cから電圧/電流/電力を取得
                Vol = (uint32)InquireI2C(I2C_SLAVE_BUSVOL) * 1250 / 1000;
                Amp = (uint16)InquireI2C(I2C_SLAVE_CURRENT);
                if  (Amp > 32768)   Amp = 0.0;
                Pow = (uint32)InquireI2C(I2C_SLAVE_POWER) * 2500 / 1000;

                // 文字列に変換
                Int2String(CharBuf, Vol);
                Int2String(CharBuf2, Amp);
                Int2String(CharBuf3, Pow);

                LCD_ClearDisplay();
                LCD_Position(0u, 0u);
                LCD_PrintString("V:");
            	LCD_Position(0u, 2u);
                LCD_PrintString(CharBuf);
                LCD_Position(0u, 8u);
                LCD_PrintString("A:");
            	LCD_Position(0u, 10u);
                LCD_PrintString(CharBuf2);
                LCD_Position(1u, 0u);
                LCD_PrintString("Power:");
            	LCD_Position(1u, 6u);
                LCD_PrintString(CharBuf3);
                if ( UARTMode )                 
                {
                    // UARTに出力
                	UART_UartPutString(CharBuf);
                    UART_UartPutString(":");
                	UART_UartPutString(CharBuf2);
                    UART_UartPutString(":");
                	UART_UartPutString(CharBuf3);
                    UART_UartPutCRLF(0);   // CR+LF
                }                
                break;
             default:
                LCD_ClearDisplay();
                break;
        }

        // Baseline update
        CapSense_UpdateEnabledBaselines();
        
   		// CapSenseのWidgetをEnable
    	CapSense_ScanEnabledWidgets();
    
        // CapSenseのスキャン待ち
		while(CapSense_IsBusy() != 0);

        // Buttonチェック
    	if (CapSense_CheckIsWidgetActive(CapSense_SENSOR_BUTTON0__BTN)) 
        {
            LED1_Write(1u);
            Mode = 0;
            UARTMode = 0;
        }
        else
            LED1_Write(0u);
        
        if (CapSense_CheckIsWidgetActive(CapSense_SENSOR_BUTTON1__BTN)) 
            Mode = 1;

        if (CapSense_CheckIsWidgetActive(CapSense_SENSOR_BUTTON2__BTN)) 
            Mode = 2;

        if (CapSense_CheckIsWidgetActive(CapSense_SENSOR_BUTTON3__BTN))
        {
            if ( UARTMode == 0 )
                UARTMode = 1;
            else
                UARTMode = 0;
        }

        // LED2/3の点灯制御
        switch( Mode )
        {
            case 1:
                LED2_Write(1u);
                LED3_Write(0u);
                break;
            case 2:
                LED2_Write(0u);
                LED3_Write(1u);
                break;
            default:
                LED2_Write(0u);
                LED3_Write(0u);
        }
    
        // LED4の点灯制御
        if ( UARTMode == 1 )
            LED4_Write(1u);
        else
            LED4_Write(0u);

        // Delay
        CyDelay(50);
    }
}

/* [] END OF FILE */
